/* Copyright (C) 2000-2002 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#include "udm_config.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include "udm_common.h"
#include "udm_mutex.h"

/*#define DEBUG_LOCK*/

static udm_mutex_t mutex[UDM_LOCK_MAX];

void UdmInitMutexes(void){
	int i;
	for(i=0;i<UDM_LOCK_MAX;i++){
		InitMutex(&mutex[i]);
	}
}

void UdmDestroyMutexes(void) {
	int i;
	for(i=0;i<UDM_LOCK_MAX;i++){
		DestroyMutex(&mutex[i]);
	}
}

/* CALL-BACK Locking function */
__INDLIB__ void UdmLockProc(UDM_AGENT *A,int command,int type,const char *fn,int ln){
#ifdef DEBUG_LOCK
	fprintf(stderr,"[%d] Try %s\t%d\t%s\t%d\n",
		A?A->handle:-1,(command==UDM_LOCK)?"lock":"unlock",
		type,fn,ln);
#endif
	switch(command){
		case UDM_LOCK:
			UDM_MUTEX_LOCK(&mutex[type]);
			break;
		case UDM_UNLOCK:
			UDM_MUTEX_UNLOCK(&mutex[type]);
			break;
	}
#ifdef DEBUG_LOCK
	fprintf(stderr,"[%d] %s\t%d\t%s\t%d\n",
		A?A->handle:-1,(command==UDM_LOCK)?"locked":"unlocked",
		type,fn,ln);
#endif
}

__INDLIB__ int UdmSetLockProc(UDM_ENV * Conf,
		void (*proc)(UDM_AGENT *A,int command,int type,const char *f,int l)){
	
	Conf->LockProc=proc;
	return(0);
}

